# coding: utf-8
import datetime

from django.contrib import admin
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django import forms
from django.utils.encoding import force_unicode
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.contrib.auth.models import User
from django.forms.models import ModelChoiceField
from django.conf.urls.defaults import patterns, url

from jumbo.admin.options import ExtendibleModelAdmin, OrderedModelAdmin, SettingsAdmin
from jumbo.forms.models import AdminModelForm
import jumbo.middleware.thread_local as tl
from jumbo.core.urlresolvers import reverse

from timereport.models import Employee, Transit, TimeProfile, Title, Settings, TransitLog, Period, Reason
from timereport import views as timereport_views
from timereport import settings as timereport_settings
from timereport import views_report as timereport_views_report

admin.site.register(Settings, SettingsAdmin)

class TimeProfileInlineAdminForm(AdminModelForm):
    class Meta:
        model = TimeProfile
class TimeProfileInline(admin.TabularInline):
    class Meta:
        verbose_name = _('time profile')
        verbose_name_plural = _('time profiles')
    model = TimeProfile
    extra = 7
    form = TimeProfileInlineAdminForm
    fieldsets = (
        (_('code admin'), {'fields': (
            'weekday','time_in', 'time_out', 'val_date_begin',
            'val_date_end', 'delta_advance', 'delta_delay'
        ),}),
    )

class UserChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        try:
            return "%s --- %s" % (obj.contact.organization, obj.contact)
        except:
            return None

class EmployeeAdminForm(AdminModelForm):
    class Meta:
        model = Employee

    def __init__(self, *args, **kws):
        super(EmployeeAdminForm, self).__init__(*args, **kws)
        user = self.get_object_id('user')
        employees = Employee.objects.exclude(user=user)
        queryset = User.objects.filter(
            is_active = True,
            contact__status=1,
            contact__organization__status=1,
            contact__organization__organization_type__category="com"
        ).exclude(employee__in=employees)
        self.fields['user'] = UserChoiceField(
            queryset=queryset, required=True
        )

class EmployeeAdmin(ExtendibleModelAdmin):
    class Media:
        css = {"all": ("timereport/css/admin_timereport.css",)}
    save_on_top = True
    form = EmployeeAdminForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify','status'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': (
                ('user', 'badge',), ('title', 'office',),
                ('vat_number', 'code_tax',),
                ('cost', 'extra_cost',),
                ('phone', 'mobile'), 'note',
            )
        }),
    )
    inlines = (TimeProfileInline,)
    list_display = ('get_employee', 'badge', 'get_detail_icon', 'get_edit_icon')
    list_display_csv = ('get_employee', 'badge', 'cost', 'extra_cost')
    list_per_page = 30

    def get_employee(self, obj):
        try:
            return "%s --- %s" % (
                obj.user.contact.organization,
                obj.user.contact
            )
        except:
            return obj.user

admin.site.register(Employee, EmployeeAdmin)


class TitleAdminForm(AdminModelForm):
    class Meta:
        model = Title

class TitleAdmin(ExtendibleModelAdmin):
    save_on_top = True
    form = TitleAdminForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify','status'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('name',)
        }),
    )
    list_display = ('name',)
    list_display_links = ('name',)
    list_per_page = 30
admin.site.register(Title, TitleAdmin)

class ReasonAdmin(ExtendibleModelAdmin):
    save_on_top = True
#    fields = ('date_create','code','description', 'status')
    list_display = ('code','description', 'status', 'color', 'in_legend',
                    'approval_required', 'get_edit_icon', 'odv')
    list_editable = ('status', 'description', 'in_legend', 'color', 'approval_required', 'odv')
    list_per_page = 30
admin.site.register(Reason, ReasonAdmin)

class TransitInline(admin.TabularInline):
    model = Transit
    fields = ('date_time_in', 'date_time_out', 'status', 'creator')
    
class PeriodAdmin(ExtendibleModelAdmin):

    class Meta:
        model = Period

    fields = ('employee', 'status')
    readonly_fields = ('employee',)
    list_display = ('employee', 'date_start', 'date_end', 'status',
                    'date_approved1', 'approved1_by', 'get_edit_icon')
    list_display_links = ('employee','approved1_by')
    list_editable = ('status',)
    list_filter = ('status', 'employee',)
    search_fields = ('employee__user__username', )
    list_per_page = 30
    def get_urls(self):

        urls = super(PeriodAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^add_vacation/$',
                self.admin_site.admin_view(timereport_views.period_add_vacation),
                name='timereport_period_add_vacation'),
                           )
        return my_urls + urls

    def save_model(self, request, obj, form, change):
        """track user that approves and date
        """
        obj.approved1_by = request.user
        obj.date_approved1 = datetime.date.today()
        obj.save()

    # def has_delete_permission(self, request, obj=None):
    #     if request.user.has_perm('timereport.can_insert_all_manual_transit'):
    #         return True
    #     if request.user.has_perm('timereport.can_insert_own_manual_transit') and (
    #         not timereport_settings.TIMEREPORT_NEED_APPROVAL1 or not obj.status == 1):
    #         # se li può inserire li può anche cancellare...
    #         return True
    #     return False


admin.site.register(Period, PeriodAdmin)


class TransitAdminForm(AdminModelForm):
    class Meta:
        model = Transit

    def __init__(self, *args, **kws):
        super(TransitAdminForm, self).__init__(*args, **kws)
        user = tl.get_current_user()
        e = Employee.active_objects.all()
        if not user.has_perm("timereport.can_insert_all_manual_transit"):
            e = e.filter(user=user)
        self.fields['employee'].queryset = e

class TransitAdmin(ExtendibleModelAdmin):
    save_on_top = True
    form = TransitAdminForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create','last_modifier', 'date_last_modify','status'),
            'classes': ('collapse',)
        }),
        (None, {
            'fields': ('employee', 'date_time_in', 'date_time_out',)
        }),
    )
    list_display = ('employee', 'date_time_in', 'date_time_out', 'duration')
    list_display_csv = ('employee', 'date_time_in', 'date_time_out', 'duration')
    list_display_links = ('employee',)
    list_per_page = 30

    def has_add_permission(self, request):
        original_add_perm = super(TransitAdmin, self).has_add_permission(request)
        if original_add_perm or request.user.has_perm("timereport.can_insert_own_manual_transit") or \
                                  request.user.has_perm("timereport.can_insert_all_manual_transit"):
            return True
        return False


    def save_model(self, request, obj, form, change):
        """
        Given a model instance save it to the database.
        """
        obj.save()
        tl = TransitLog()
        tl.employee = obj.employee
        tl.date_time_in = obj.date_time_in
        tl.date_time_out = obj.date_time_out
        tl.entry_type = 'M'
        tl.save()


    def get_urls(self):
        from django.conf.urls.defaults import patterns, url
        urls = super(TransitAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^$'                                 , 'jumbo.apps.calendar.views.calendar_list'
                                                      , {
                                                            'model' : Transit,
                                                            'field_date_begin': 'date_time_in',
                                                            'field_date_end': 'date_time_out',
                                                            'template_dir':'admin/timereport/calendar'
                                                        }
                                                      , name='timereport_transit_calendar'),

            url(r'^calendar/$'                        , 'jumbo.apps.calendar.views.calendar_list'
                                                      , {
                                                            'model' : Transit,
                                                            'field_date_begin': 'date_time_in',
                                                            'field_date_end': 'date_time_out',
                                                            'template_dir':'admin/timereport/calendar'
                                                        }
                                                      , name='timereport_transit_calendar'),

            url(r'^calendar/(?P<type_list>\w+)/$'     , 'jumbo.apps.calendar.views.calendar_list'
                                                      , {
                                                            'model' : Transit,
                                                            'field_date_begin': 'date_time_in',
                                                            'field_date_end': 'date_time_out',
                                                            'template_dir':'admin/timereport/calendar'
                                                        }
                                                      , name='timereport_transit_calendar'),

            url(r'^calendar/(?P<type_list>\w+)/(?P<day>\d+)/$', 'jumbo.apps.calendar.views.calendar_list'
                                                              , {
                                                                    'model' : Transit,
                                                                    'field_date_begin': 'date_time_in',
                                                                    'field_date_end': 'date_time_out',
                                                                    'template_dir':'admin/timereport/calendar'
                                                                }
                                                              , name='timereport_transit_calendar'),

            url(r'^add_visual/$', self.admin_site.admin_view(timereport_views.transit_add_visual)
                                , name='timereport_transit_add_visual'),

            url(r'^add_permit/$', self.admin_site.admin_view(timereport_views.transit_add_permit)
                                , name='timereport_transit_add_permit'),
            # url(r'^add_vacation/$', self.admin_site.admin_view(timereport_views.period_add_vacation)
            #                     , name='timereport_period_add_vacation'),
            url(r'^add_illness/$', self.admin_site.admin_view(timereport_views.transit_add_illness)
                                , name='timereport_transit_add_illness'),
            url(r'^add_injury/$', self.admin_site.admin_view(timereport_views.transit_add_injury)
                                , name='timereport_transit_add_injury'),
            url(r'^download/$'  , self.admin_site.admin_view(timereport_views_report.transit_download)
                                , name='timereport_transit_download',),
        )
        return my_urls + urls


class TransitLogSearchForm(forms.ModelForm):
    class Meta:
        model = TransitLog
        display_fields = (
            ('employee', 'entry_type'),
        )

class TransitLogAdmin(ExtendibleModelAdmin):
    list_display = ('date_create','creator', 'employee', 'date_time_in', 'date_time_out', 'entry_type')
    search_form = TransitLogSearchForm
    search_fields = ('employee__user__last_name',)


admin.site.register(Transit, TransitAdmin)
admin.site.register(TransitLog, TransitLogAdmin)
