# coding: utf-8
from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from datetime import date, time, datetime, timedelta
from django.core.mail import EmailMessage

from jumbo.forms.models import ModelForm
import jumbo.middleware.thread_local as tl
from jumbo.utils.functions import get_settings
from jumbo.forms import DateWidget, DateTimeWidget

from timereport.models import Employee, Transit, Period, STATUS_CODES, Reason
from timereport import settings as timereport_settings

class Transito(ModelForm):

    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out',)

    def __init__(self,  *args, **kwargs):
        super(Transito, self).__init__(*args, **kwargs)
        self.fields['employee'].widget.choices = self.get_employee()
        self.fields['date_time_in'].widget= DateTimeWidget()
        self.fields['date_time_out'].widget= DateTimeWidget()


    def get_employee(self):
        user = tl.get_current_user()

        query = Employee.active_objects.all()

        if user.has_perm("timereport.can_insert_all_manual_transit"):
            query = query
        else:
            query = query.filter(user=user)

        query = query.order_by('user__last_name')

        for i in query:
            yield (i.id, i.name)


class TransitForm(Transito):
    def save(self):
        self.instance.status = 21
        super(TransitForm, self).save()

class PermitForm(Transito):
    def __init__(self, *args, **kwargs):
        super(PermitForm, self).__init__(*args, **kwargs)
        self.fields['date_time_in'].label = _("Start permit")
        self.fields['date_time_out'].label = _("Stop permit")

    class Meta:
        model = Transit
        fields = ('employee', 'date_time_in', 'date_time_out', 'note')

    def save(self):
        self.instance.status = 22
        super(PermitForm, self).save()
        self.instance.send_mail()

class VacationPeriodForm(forms.Form):
    REASON = 23  # VACATION nei models
    start_date = forms.DateField()
    end_date   = forms.DateField()
    employee   = forms.ChoiceField()
    reason     = forms.ChoiceField()
    approved   = forms.BooleanField(required=False)

    def __init__(self, *args, **kwargs):
        super(VacationPeriodForm, self).__init__(*args, **kwargs)

        self.fields['reason'].choices = self.get_reasons()
        self.fields['employee'].choices = self.get_employee()

        self.fields['start_date'] = forms.DateField(label=_("Start Date"), widget= DateWidget())
        self.fields['start_time'] = forms.TimeField(label=_("Start Time"), required=False)

        self.fields['end_date'] = forms.DateField(label=_("End date"), widget= DateWidget())
        self.fields['end_time'] = forms.TimeField(label=_("End time"), required=False)

    def get_employee(self):
        """Ritorna generatore per l'elenco di utenti per cui
        current user può inserire perdiodi di ferie.
        usa ``timereport.can_insert_all_manual_transit``
        """
        
        query = Employee.active_objects.all()
        user = tl.get_current_user()

        if user.has_perm("timereport.can_insert_all_manual_transit"):
            query = query
        else:
            query = query.filter(user=user)

        query = query.order_by('user__last_name')

        for i in query.all():
            yield (i.id, i.name)

    def get_reasons(self):
        """Ritorna generatore per l'elenco di causali 
        """
        
        query = Reason.objects.filter(status=1).order_by('odv')
        for r in query:
            yield (r.code, r.description)

    def clean(self):
        if not self.cleaned_data['end_date']:
            self.cleaned_data['end_date'] = self.cleaned_data['start_date']

        if self.cleaned_data['end_time'] and not self.cleaned_data['start_time']:
            raise ValidationError(_('Missing start time'))

        if self.cleaned_data['start_time'] and not self.cleaned_data['end_time']:
            raise ValidationError(_('Missing end time'))

        if not self.cleaned_data['end_date'] == self.cleaned_data['start_date']:
            if self.cleaned_data['start_time'] or self.cleaned_data['end_time']:
                raise ValidationError(_("Don't use time when dates are different"))  
        return self.cleaned_data

    def save(self):
        ""
        period = Period(status=0, employee_id=self.cleaned_data['employee'],)
        # vedi la proprietà start_date del modello per capire il giro...
        period._start_date = self.cleaned_data['start_date']
        period._end_date = self.cleaned_data['end_date']
        period._reason = Reason.objects.filter(code=self.cleaned_data['reason'])[0]
        period.status = 0 if period._reason.approval_required else 1
        period.save()

        for day in self.get_working_days():
            date_time_in = datetime.combine(day, self.cleaned_data['start_time'] or time(9,0))

            if self.cleaned_data['end_time']:
                date_time_out = datetime.combine(day, self.cleaned_data['end_time'])
            else:
                date_time_out = date_time_in + timedelta(hours=8)

            if not self.cleaned_data['end_time']:
                # sono possibili permessi ad ore
                date_time_out = datetime.combine(
                    day, self.cleaned_data['start_time'] or time(9,0)) + timedelta(hours=8)
            
            t = Transit(reason = period._reason,
                        status = 100, # Not used
                        employee_id = self.cleaned_data['employee'],
                        date_time_in = date_time_in,
                        date_time_out = date_time_out,
                        accepted = bool(period.status),
                        period = period,
                    )
            t.save()

    def get_working_days(self):
        """return a list of date.date of all working days from 'start' to 'end' 
        FIXME: dovrebbe considerare vacanze e profilo
        """
        days = []
        date = self.cleaned_data['start_date']
        if not self.cleaned_data['end_date']:
            self.cleaned_data['end_date'] = self.cleaned_data['start_date']
            
        while date <=  self.cleaned_data['end_date']:
            if date.isoweekday() <= 5:  # lun-ven FIXME: dovremmo usare il profilo!
                days += [date]
            date += timedelta(days=1)
        return days

# class IllnessForm(Transito):
#     class Meta:
#         model = Transit
#         fields = ('employee', 'date_time_in', 'date_time_out',)

#     def __init__(self, *args, **kwargs):
#         super(IllnessForm, self).__init__(*args, **kwargs)
#         self.fields['date_time_in'] = forms.DateField(label=_("Illness Date From"), widget= DateWidget())
#         self.fields['date_time_out'] = forms.DateField(label=_("Illness Date To"), widget= DateWidget())


#     def clean_date_time_in(self):
#         self.cleaned_data['date_time_in'] = datetime.combine(
#                 self.cleaned_data['date_time_in'],
#                 time().min
#         )
#         return self.cleaned_data['date_time_in']

#     def clean_date_time_out(self):
#         self.cleaned_data['date_time_out'] = datetime.combine(
#                 self.cleaned_data['date_time_out'],
#                 time(23, 59, 59)
#         )
#         return self.cleaned_data['date_time_out']

#     def save(self):
#         self.instance.status = 25
#         super(IllnessForm, self).save()
#         self.instance.send_mail()


# class InjuryForm(Transito):
#     class Meta:
#         model = Transit
#         fields = ('employee', 'date_time_in', 'date_time_out',)

#     def __init__(self, *args, **kwargs):
#         super(InjuryForm, self).__init__(*args, **kwargs)
#         self.fields['date_time_in'] = forms.DateField(label=_("Date From"), widget= DateWidget())
#         self.fields['date_time_out'] = forms.DateField(label=_("Date To"), widget= DateWidget())

#     def clean_date_time_in(self):
#         self.cleaned_data['date_time_in'] = datetime.combine(
#                 self.cleaned_data['date_time_in'],
#                 time().min
#         )
#         return self.cleaned_data['date_time_in']

#     def clean_date_time_out(self):
#         self.cleaned_data['date_time_out'] = datetime.combine(
#                 self.cleaned_data['date_time_out'],
#                 time(23, 59, 59)
#         )
#         return self.cleaned_data['date_time_out']

#     def save(self):
#         self.instance.status = 26
#         super(InjuryForm, self).save()
#         self.send_mail()


class DownloadTransitForm(forms.Form):
    date_begin = forms.DateField(widget=DateWidget)
    date_end = forms.DateField(widget=DateWidget)
