# -*- coding: utf-8 -*-
"""
Models
======

Ecco i modelli definiti

.. autoclass:: Employee
   :members: active_objects, is_present

.. autoclass:: Transit

.. autoclass:: Period

"""
from datetime import date, time, timedelta, datetime

from django.db import models, connection
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext
from django.contrib.auth.models import User
from django.core.mail import EmailMessage
from django.conf.urls.defaults import url
from django.core.urlresolvers import reverse
from django.db.models.signals import post_save

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.db.models import Settings as JSettings
from jumbo.middleware.thread_local import get_current_user
from jumbo.utils.functions import get_formatted_time, get_settings
from jumbo.shortcuts import render_to_response, render_to_string

from organization.models import Office

from timereport.utils import Duration
from timereport import settings
from timereport.exception import TransitException


class ActiveEmployee(models.Manager):
    def get_query_set(self):
        return super(ActiveEmployee, self).get_query_set().filter(status=1)

class Employee(DateModel, UserModel, StatusModel):
    """
    Dipendenti. Oltre al default objects ha il manager :attr:`active_objects`
    """
    ## ForeignKey
    user = models.ForeignKey(
        User,
        related_name = 'employee',
        unique = True,
        db_index = True,
        verbose_name = _('User'),
    )

    office = models.ForeignKey(
        Office,
        related_name = 'employees',
        db_index = True,
        verbose_name = _('Office')
    )

    title = models.ForeignKey(
        "Title",
        related_name = 'employees',
        db_index = True,
        verbose_name = _('Title'),
    )

    ## Campi Modello
    badge = models.CharField(
        max_length = 15,
        verbose_name = _('Badge Number')
    )

    vat_number = models.CharField (
        max_length = 11,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Vat Number"),
    )

    code_tax = models.CharField (
        max_length=16,
        blank = True, null = True,
        db_index = True,
        verbose_name = _("Code Tax"),
    )

    phone = models.CharField (
        max_length = 20,
        verbose_name = _("Phone"),
    )

    mobile = models.CharField (
        blank = True, null = True,
        max_length = 20,
        verbose_name = _("Mobile"),
    )

    note = models.TextField (
        blank = True, null = True,
        verbose_name = _("Note"),
    )

    cost = models.DecimalField (
        blank = True, null = True,
        max_digits = 5,
        decimal_places = 2,
        verbose_name = _("Cost per hour"),
    )
    extra_cost = models.DecimalField (
        blank = True, null = True,
        max_digits = 5,
        decimal_places = 2,
        verbose_name = _("Extra Cost per hour"),
    )

    objects = models.Manager() # The default manager.
    #: Manager che ritorna solo i dipendenti attivi
    active_objects = ActiveEmployee() # The active-employee manager.

    class Meta:
        verbose_name = _('Employee')
        verbose_name_plural = _('Employees')
        permissions = (
            ("view_employee"                   , "Can view employee"),
        )

    def add_in_transit(self):
        t = Transit()
        t.employee = self
        t.date_time_in = datetime.combine(datetime.today().date(), time(datetime.today().hour, datetime.today().minute))

        try:
            t.save()
            tl = TransitLog()
            tl.employee = t.employee
            tl.date_time_in = t.date_time_in
            tl.entry_type = 'A'
            tl.save()
        except TransitException, e:
            raise TransitException(e.value)


    def add_out_transit(self):
        t = Transit.objects.filter(employee=self.id, date_time_out__isnull=True)
        if not t:
            raise TransitException(ugettext("Missing entrance datetime"))
        if len(t) > 1:
            raise TransitException(ugettext("Multiple entrance with no exit datetime"))

        t = t[0]
        t.date_time_out = datetime.combine(datetime.today().date(), time(datetime.today().hour, datetime.today().minute))
        try:
            t.save()
            tl = TransitLog()
            tl.employee = t.employee
            tl.date_time_out = t.date_time_out
            tl.entry_type = 'A'
            tl.save()
        except TransitException, e:
            raise TransitException(e.value)


    def get_name(self):
        n = "%s %s" % (self.user.last_name , self.user.first_name)
        return n.strip() or self.user.username
    name = property(get_name)

    def get_small_name(self):
        n = "%s %s" % (self.user.last_name , self.user.first_name[0:1])
        return n.strip() or self.user.username
    small_name = property(get_small_name)

    def is_present(self):
        """
        Return True se il dipendente ha, per la data odierna, una entrata e nessuna uscita.
        """
        try:
            date_tr = date.today()
            is_present=None
            transits = self.transits.filter(date_time_in__year=date_tr.year, date_time_in__month=date_tr.month, date_time_in__day=date_tr.day)
            if transits:
                is_present=False
                transit=transits.filter(date_time_out__isnull=True)
                if transit:
                    is_present=True
            return is_present
        except:
            return False

    def get_status_absence(self, date_tr=date.today):
        transits=self.transits.filter(date_time_in__year=date_tr.year, date_time_in__month=date_tr.month, date_time_in__day=date_tr.day)
        if transits.filter(status=23):
            return 'F'
        elif transits.filter(status=25):
            return 'M'
        elif transits.filter(status=26):
            return 'I'
        return False

    def get_total_transit_time(self, date_tr=date.today()):
        transits=self.transits.filter(date_time_in__year=date_tr.year, date_time_in__month=date_tr.month, date_time_in__day=date_tr.day,
                                      status=21)
        total_time=timedelta()
        if transits:
            for tr in transits:
                if tr.duration:
                    total_time+=tr.duration
        return total_time

    def get_total_time(self, date_tr=date.today()):
        transits=self.transits.filter(date_time_in__year=date_tr.year, date_time_in__month=date_tr.month, date_time_in__day=date_tr.day)
        total_time=timedelta()
        if transits:
            for tr in transits:
                if tr.duration:
                    total_time+=tr.duration
        return total_time

    def get_totals_time(self, date_tr=date.today()):
        transits=self.transits.filter(date_time_in__year=date_tr.year, date_time_in__month=date_tr.month, date_time_in__day=date_tr.day)
        d = {21: Duration(21), 22: Duration(22), 23: Duration(23), 24: Duration(24), 25: Duration(25), 26: Duration(26)}

        if transits:
            for tr in transits:
                if tr.duration:
                    d[tr.status].duration += tr.duration
        return d.values()


    def get_ordinary_duration(self, date_ord=date.today()):
        duration = timedelta(0)
        weekday = date_ord.weekday()
        timeprofiles = self.timeprofiles.filter(
            weekday=weekday,
            val_date_begin__lte=date_ord,
            val_date_end__gte=date_ord
        )

        for tp in timeprofiles:
            duration += tp.duration

        return duration

    def save(self):
        ### controllo che non ci sia un altro utente con lo stesso badge 'ATTIVO'
        emps = Employee.objects.filter(status=1, badge=self.badge).exclude(id=self.id)[:0]
        if not emps:
            super(Employee, self).save()

    def __unicode__(self):
        return u"%s - %s" % (self.get_name(), self.badge)

################################
# TRANSITO                     #
################################
STATUS_CODES = (
    ( 21, _('Transit')),
    ( 22, _('Permit')),
    ( 23, _('Vacation')),
    ( 24, _('Permit Not Paid')),
    ( 25, _('Illness')),
    ( 26, _('Injury')),
    (100, _('Not Used')),  # Dopo introduzione di reason
)

class Title(DateModel, StatusModel, UserModel):
    ## Campi Modello
    name = models.CharField (
        max_length = 50,
        db_index = True,
        unique = True,
        verbose_name = _("Name"),
    )

    def __unicode__(self):
        return u'%s' % (self.name,)


class TransitManager(models.Manager):
    def get_query_set(self):
        qs = super(TransitManager, self).get_query_set()
        user = get_current_user()

        # se sono l'utente anonimo (l'utente anonimo ha la possibilita)
        # di visualizzare solo la pagina di inserimento transito) allora
        # prendo tt i transiti per permettergli la possibilita di poter
        # modificare l'orario di uscita
        # tolta da marco nel 2010, nel 2012 sandro la rimette in attesa
        # di soluzione migliore per evitare errore visibile anche solo
        # da shell::
        #   from jumbo.conf.urls import JUMBO_URL
        
        if user.is_anonymous():
            return qs   # FIXME
        if user.has_perm("timereport.view_all_transit"):
            return qs
        return qs.filter(employee__user=user)

class Transit(DateModel, UserModel):
    """
    Un transito è definito da una coppia di datetime entrata/uscita di un :class:`employee`

    """
    ## Campi generici
    status = models.IntegerField (
        default = 21, # overloaded by reason_id
        choices = STATUS_CODES,
        db_index = True,
        verbose_name = _('Status'),
        null=False, blank=True,
    )
    ## ForeignKey
    employee = models.ForeignKey(
        Employee,
        related_name = 'transits',
        db_index = True,
        verbose_name = _('Employee')
    )
    ## Campi Modello
    date_time_in = models.DateTimeField (
        verbose_name = _('Entry Date'),
    )
    date_time_out = models.DateTimeField (
        null = True, blank = True,
        verbose_name = _('Exit Date'),
    )
    accepted = models.NullBooleanField(
        null = True, blank = True,
        db_index = True, default=False,
        verbose_name = _('Accepted'),
    )
    note = models.TextField(
        null = True, blank=True,
        verbose_name = _('Description'),
        help_text = _("Permit Request's Reason. Not Required.")
    )
    all_objects = models.Manager()
    objects = TransitManager()

    ## Campi generici
    period = models.ForeignKey ('Period', related_name='transits', null=True, blank=True)
    reason = models.ForeignKey ('Reason', null=True, blank=True, default='T')

    def get_duration(self):
        ## differenza tra uscita e ingresso
        if not self.date_time_out or not self.date_time_in:
            return None
        return self.date_time_out - self.date_time_in
    duration = property(get_duration)

    def check_transit_error(self):
        ### verifico che l'oggetto si adi tipo 'Transito' - necessario per le chiamate da shell
        if self.status != 21:
            return

        ## verifico che ENTRATA non sia s8uccessiva a NOW
        if self.date_time_in and self.date_time_in > datetime.now():
            raise TransitException(ugettext("ERROR. FUTURE ENTRY"))

        ## verifico che USCITA non sia successiva a NOW
        if self.date_time_out and self.date_time_out > datetime.now():
            raise TransitException(ugettext("ERROR. FUTURE EXIT"))

        ## controllo che non ci sia un transito 'aperto'
        transits = Transit.objects.filter(employee=self.employee, date_time_out__isnull=True).exclude(id=self.id)
        if transits:
            raise TransitException(ugettext("ENTRY ALREADY DONE."))


    def check_other_error(self):
        #if self.status == 21:
        #    return
        pass

    def check_global_error(self):
        ## verifico che USCITA è successiva all'ENTRATA
        if self.date_time_out and self.date_time_in >= self.date_time_out:
            raise TransitException(ugettext("EXIT DATE EARLIER THEN ENTRY DATE."))

        ###################################
        ## VERIFICA SOVRAPPOSIZIONI
        ###################################

        #escludo prima il transito che sto modificando altrimenti mi si sovrappone sicuramente)
        transits = Transit.objects.exclude(pk=self.id)

        # Sto facendo un'ENTRATA
        if not self.date_time_out:
            transits = transits.filter(employee=self.employee, date_time_in__lte=self.date_time_in, date_time_out__gte=self.date_time_in)

        # Sto facendo un'USCITA
        else:
            transits = transits.filter(employee=self.employee, date_time_out__gte=self.date_time_in, date_time_in__lte=self.date_time_out)

        if transits:
            raise TransitException(ugettext("ERROR. OVERLAP DATES."))

        ###################################
        ## FINE VERIFICA SOVRAPPOSIZIONI
        ###################################

    def send_mail(self):
        recipient_list = get_settings("timereport.settings", "TIMEREPORT_TO_MAIL")
        from_mail = get_settings("timereport.settings", "TIMEREPORT_FROM_MAIL")

        subject = render_to_string("admin/timereport/transit/mail_transit_subject.txt", tr=self)
        body_message = render_to_string("admin/timereport/transit/mail_transit_body.txt", tr=self)

        if recipient_list:
            email = EmailMessage(subject, body_message, from_mail, recipient_list)
            email.send()

    def save(self,*args, **kwargs):
        user = get_current_user()

        # Se l'utente puo inserire/modificare tutti i transiti vuol dire che e' un amministratore
        # del timereport e quindi evito di fare i vari controlli, altrimenti andasse a modificare
        # un vecchio transito darebbe errore ("ENTRATA GIA EFFETTUATA")
        #
        if not user.has_perm("timereport.change_transit") and \
           not user.has_perm("timereport.can_insert_all_manual_transit"):
            ## verifico vari check
            self.check_transit_error()
            self.check_other_error()
            self.check_global_error()
            # evito che un utente che non ha i permessi possa aggiungere permessi/malattie/ecc
            # di altri utenti
            if not user.has_perm("timereport.view_all_transit"):
                self.employee = user.employee.get()

        ## nel caso in cui il giorno di USCITA/FINE è diverso dal giorno di ENTRATA/INIZIO
        if self.date_time_out:
            try:
                if self.date_time_out.date() > self.date_time_in.date():
                    #self.date_time_out.date() > self.date_time_in.date()
                    ## modifico ora USCITA/FINE alla MEZZANOTTE del giorno di ENTRATA/INIZIO
                    date_time_out = self.date_time_out
                    self.date_time_out = datetime.combine(
                        self.date_time_in.date(),
                        time(23, 59, 59)
                    )
                    super(Transit, self).save()
                    ## creo un nuovo transito con giorno ENTRATA/INIZIO a giorno successivo e giorno USCITA/FINE ultimo
                    date_time_in = datetime.combine(
                        self.date_time_in.date() + timedelta(1),
                        time().min
                    )
                    t = Transit(
                        status=self.status, employee=self.employee,
                        date_time_in=date_time_in, date_time_out=date_time_out
                    )
                    t.save()

            except Exception, e:
                # print e
                pass
        super(Transit, self).save()

    class Meta:
        verbose_name = _('Transit')
        verbose_name_plural = _('Transits')
        ordering = ('employee__user__last_name', 'date_time_in',)
        permissions = (
            ("view_transit"                   , "Can view transit"),
            ("view_all_transit"               , "Can view ALL TRANSIT"),
            ("can_insert_own_manual_transit"  , "Can insert OWN MANUAL TRANSIT"),
            ("can_insert_all_manual_transit"  , "Can insert ALL MANUAL TRANSIT"),
            ("can_download_report_transit"    , "Can DOWNLOAD REPORT TRANSIT"),
        )

    def __unicode__(self):
        duration = self.duration
        return u"%s : %s - %s -> %s" % (self.employee.get_name(), self.date_time_in, self.date_time_out, duration.__str__())

PERIOD_CHOICES = (
    (-1, _('rejected')),
    (0, _('pending')),
    (1, _('accepted')),
    )

class Period(DateModel):
    """
    Aggregatore di transiti. Usato per accettare/respingere/cancellare blocchi di
    transiti per ferie/permessi
    """
    status         = models.IntegerField(choices=PERIOD_CHOICES, default=0)
    description    = models.CharField(max_length=100, null=True, blank=True)
    employee       =  models.ForeignKey(Employee, related_name = 'period',
                                        verbose_name = _('Employee'), null=True)

    # start_date     = models.DateField()
    # end_date       = models.DateField()
    date_approved1 = models.DateField(null=True, blank=True, verbose_name = _('Approved on'),)
    approved1_by   = models.ForeignKey(User, null=True, blank=True,
                                       verbose_name = _('Approved by'),related_name = 'approved1')
    date_approved2 = models.DateField(null=True, blank=True)
    approved2_by   = models.ForeignKey(User, null=True, blank=True,
                                     related_name = 'approved2')
    objects = TransitManager()  # Manager identico al Transit

    @property
    def date_start(self):
        # quando viene inviata la mail, i transiti non sono ancora stati
        # salvati, il post_save viene inviato nella prima fase
        try:
            date = self._start_date
        except AttributeError:
            try:
                date = self.transits.all().order_by('date_time_in')[0].date_time_in.date()
            except IndexError:
                date = None
        return date and date.strftime('%d/%m/%Y')

    @property
    def date_end(self):
        try:
            date = self._end_date
        except AttributeError:
            try:
                date = self.transits.all().order_by('-date_time_out')[0].date_time_out.date()
            except IndexError:
                date = None

        return date and date.strftime('%d/%m/%Y')

    @property
    def reason(self):
        "Return a :class:`Reason` instance related to the transits in this Period"
        try:
            reason = self._reason
        except AttributeError:
            try:
                reason = self.transits.all().order_by('-date_time_out')[0].reason
            except IndexError:
                reason = None

        return reason 

    def get_approve_url(self):
        
        return "%s%s" % (getattr(settings, 'TIMEREPORT_HOST', 'http://TIMEREPORT_HOST'),
                         reverse('admin:timereport_period_change', args=[self.id]))

    def get_mail_subject(self):
        "Return a subject line for the mail"
        return "%s (%s)" % (self.reason.description, self.employee.user.get_full_name())

def send_mail_vacation_period(sender, **kwargs):
    """
    Mail di avviso richiesta/approvazione vacation
    """
    from_mail = settings.TIMEREPORT_FROM_MAIL
    obj = kwargs['instance']
    if not obj.reason.approval_required:
        return

    if kwargs['created']:
        recipient_list = settings.TIMEREPORT_TO_MAIL
        if not recipient_list:
            return
        if callable(recipient_list):
            recipient_list = recipient_list(kwargs['instance'])

        subject = render_to_string("admin/timereport/transit/mail_transit_subject.txt",
                                   prd=kwargs['instance']).strip()
        body_message = render_to_string("admin/timereport/period/mail_transit_body.txt",
                                    prd=kwargs['instance'])

    else:
        recipient_list = [obj.employee.user.email]
        if obj.status == 1:
            subject = _("Your vacation request has been accepted")
        elif obj.status == 0:
            subject = _("Your vacation request has been set to pending")
        elif obj.status == -1:
            subject = _("Your vacation request has been rejected")
        body_message = render_to_string("admin/timereport/period/mail_period_body.txt",
                                    prd=kwargs['instance'])
        
    if recipient_list:
        email = EmailMessage(subject, body_message, from_mail, recipient_list)
        email.send()
        

post_save.connect(send_mail_vacation_period, Period)

#######################################
# PROFILO CARTELLINO                  #
#######################################
WEEKDAY = (
    ( 0, _('Monday')),
    ( 1, _('Tuesday')),
    ( 2, _('Wednesday')),
    ( 3, _('Thursday')),
    ( 4, _('Friday')),
    ( 5, _('Saturday')),
    ( 6, _('Sunday')),
)

class TimeProfile(models.Model):
    ## ForeignKey
    employee = models.ForeignKey(
        Employee,
        related_name = 'timeprofiles',
        db_index = True,
        verbose_name = _('Employee')
    )
    ## Campi Modello
    weekday = models.IntegerField (
        choices = WEEKDAY,
        db_index = True,
        verbose_name = _('Week Day'),
    )
    time_in = models.TimeField (
        verbose_name = _('Time IN'),
    )

    time_out = models.TimeField (
        verbose_name = _('Time OUT'),
    )
    val_date_begin = models.DateField (
        null = True, blank = True,
        verbose_name = _("Begin Date"),
    )
    val_date_end = models.DateField (
        null = True, blank = True,
        verbose_name = _("End Date"),
    )
    ## FIXME
    delta_advance  = models.IntegerField (
        default = settings.TIMEREPORT_DELTA_ADVANCE,
        verbose_name = _("Advance Flexibility in Minutes"),
    )
    ## FIXME
    delta_delay = models.IntegerField (
        default = settings.TIMEREPORT_DELTA_DELAY,
        verbose_name = _("Delay Flexibility in Minutes"),
    )

    def get_duration(self):
        ti = timedelta(
            hours=self.time_in.hour, minutes=self.time_in.minute, seconds=self.time_in.second
        )
        to = timedelta(
            hours=self.time_out.hour, minutes=self.time_out.minute, seconds=self.time_out.second
        )
        return to-ti
    duration = property(get_duration)

    def __unicode__(self):
        return u"%s: %s - %s %s - %s" % (
            self.get_weekday_display(),
            self.val_date_begin.strftime("%d/%m/%Y"), self.val_date_end.strftime("%d/%m/%Y"),
            self.time_in.__str__(), self.time_out.__str__()
        )

    class Meta:
        verbose_name = _('Time Profile')
        verbose_name_plural = _('Time Profiles')

    def save(self):
        ## imposto date inizio e fine periodo se non esistono
        if not self.val_date_begin:
            self.val_date_begin = date(1970, 1, 1)
        if not self.val_date_end:
            self.val_date_end = date(2050, 12, 31)
        ## verifico sovrapposizioni periodo
        tprofiles = TimeProfile.objects.filter(employee=self.employee, weekday=self.weekday, val_date_begin__lte=self.val_date_end, val_date_end__gte=self.val_date_begin)
        ## verifico sovrapposizioni orario
        tprofiles = tprofiles.filter(time_out__lte=self.time_in, time_in__gte=self.time_out)[:0]
        if not tprofiles:
            super(TimeProfile, self).save()


ENTRY_TYPE = (
    ('M', 'Manual'),
    ('A', 'Automatic'),
)
class TransitLog(DateModel, UserModel):
    ## ForeignKey
    employee = models.ForeignKey(
        Employee,
        related_name = 'transit_logs',
        db_index = True,
        verbose_name = _('Employee')
    )
    ## Campi Modello
    date_time_in = models.DateTimeField (
        null = True, blank = True,
        verbose_name = _('Entry Date'),
    )
    date_time_out = models.DateTimeField (
        null = True, blank = True,
        verbose_name = _('Exit Date'),
    )
    entry_type = models.CharField(
        max_length = 1,
        choices = ENTRY_TYPE,
        db_index = True,
        verbose_name = _('Entry Type'),
    )

class Settings(JSettings):
    pass
REASON_CODES = (
    (0, 'Inactive'),
    (1, 'Active'),
    )
class Reason(models.Model):

    """This explains the reason of the transit. A 'default' code must be
    present. May be any locally accepted reason eg.: permit, permit due to
    medical visit... 
    """
    ## Campi generici
    date_create = models.DateTimeField (auto_now_add=True,null=True)
    date_last_modify = models.DateTimeField (auto_now=True,null=True)
    status = models.IntegerField (choices=REASON_CODES, default=1)

    ##
    code = models.CharField(max_length=7, unique=True, primary_key=True)
    description = models.CharField(max_length=40)
    color = models.CharField(max_length=12, help_text=_('color used in legend'),
                             null=True, blank=True)
    in_legend = models.BooleanField(default=True,
                                    help_text=_('Should this reason appear in the legend?'))
    approval_required = models.BooleanField(default=True,
                                            help_text=_('Is an approval required?'))
    odv = models.PositiveSmallIntegerField(null=True, blank=True)

    def get_color(self, accepted):
        "return self.color_accepted or color"
        if accepted:
            return self.color_accepted or self.color
        return self.color
    
    def __unicode__(self):
        return u"%s - %s" % (self.code, self.description) 

    class Meta:
        verbose_name = _('Reason')
        verbose_name_plural = _('Reasons')
        
